@@ -2,17 +2,32 @@ matchit <- function(formula, data = NULL, method = "nearest", distance = "glm",
                     link = "logit", distance.options = list(), estimand = "ATT",
                     exact = NULL, mahvars = NULL, antiexact = NULL, discard = "none",
                     reestimate = FALSE, s.weights = NULL, replace = FALSE, m.order = NULL,
-                    caliper = NULL, std.caliper = TRUE, ratio = 1, verbose = FALSE, ...) {
+                    caliper = NULL, std.caliper = TRUE, ratio = 1, verbose = FALSE,
+                    include.obj = FALSE, ...) {
 
   #Checking input format
   #data input
   mcall <- match.call()
 
+  ## Process method
+  if (length(method) == 1 && is.character(method)) {
+    method <- tolower(method)
+    method <- match_arg(method, c("exact", "cem", "nearest", "optimal", "full", "genetic", "subclass", "cardinality"))
+    fn2 <- paste0("matchit2", method)
+  }
+  else if (is.null(method)) {
+    fn2 <- "matchit2null"
+  }
+  else {
+    stop("'method' must be the name of a supported matching method. See ?matchit for allowable options.", call. = FALSE)
+  }
+
   #Process formula and data inputs
   if (!inherits(formula, "formula")) stop("'formula' must be a formula object.", call. = FALSE)
-  f.env <- environment(formula)
 
-  treat.form <- update(terms(formula, data = data), . ~ 0)
+  tt <- terms(formula, data = data)
+  if (attr(tt, "response") != 1) stop("A treatment variable must be included on the left-hand side of 'formula'.", call. = FALSE)
+  treat.form <- update(tt, . ~ 0)
   treat.mf <- model.frame(treat.form, data = data, na.action = "na.pass")
   treat <- model.response(treat.mf)
   if (anyNA(treat)) stop("Missing values are not allowed in the treatment.", call. = FALSE)
@@ -22,6 +37,16 @@ matchit <- function(formula, data = NULL, method = "nearest", distance = "glm",
 
   n.obs <- length(treat)
 
+  #Process inputs
+  ignored.inputs <- check.inputs(mcall = mcall, method = method, distance = distance, exact = exact,
+                                 mahvars = mahvars, antiexact = antiexact, caliper = caliper, discard = discard,
+                                 reestimate = reestimate, s.weights = s.weights, replace = replace,
+                                 ratio = ratio, m.order = m.order, estimand = estimand)
+
+  if (length(ignored.inputs) > 0) {
+    for (i in ignored.inputs) assign(i, NULL)
+  }
+
   #Process s.weights
   if (!is.null(s.weights)) {
     if (is.character(s.weights)) {
@@ -53,28 +78,8 @@ matchit <- function(formula, data = NULL, method = "nearest", distance = "glm",
 
   }
 
-  ## Process method
-  if (length(method) == 1 && is.atomic(method)) {
-    method <- tolower(as.character(method))
-    fn2 <- paste0("matchit2", method)
-    if (!exists(fn2)) stop(paste0("\"", method, "\" is not a supported method."), call. = FALSE)
-  }
-  else if (is.null(method)) {
-    fn2 <- "matchit2null"
-  }
-  else {
-    stop("'method' must be the name of a supported matching method. See ?matchit for allowable options.", call. = FALSE)
-  }
-
-  #Process distance and discard
-  check.inputs(method = method, distance = distance, mcall = mcall, exact = exact,
-               mahvars = mahvars, antiexact = antiexact, caliper = caliper, discard = discard,
-               reestimate = reestimate, s.weights = s.weights, replace = replace,
-               ratio = ratio, m.order = m.order, estimand = estimand)
-
-  exactcovs <- mahcovs <- antiexactcovs <- calcovs <- NULL
-
-  if (!is.null(method) && method %in% c("exact", "cem")) {
+  #Process distance function
+  if (!is.null(method) && method %in% c("exact", "cem", "cardinality")) {
     fn1 <- NULL
   }
   else {
@@ -86,89 +91,85 @@ matchit <- function(formula, data = NULL, method = "nearest", distance = "glm",
     else {
       fn1 <- paste0("distance2", distance)
     }
-
-    if (!is.null(method)) {
-      if (!is.null(exact)) {
-        if (is.character(exact)) {
-          if (is.null(data) || !is.data.frame(data)) {
-            stop("If 'exact' is specified as strings, a data frame containing the named variables must be supplied to 'data'.", call. = FALSE)
-          }
-          if (!all(exact %in% names(data))) {
-            stop("All names supplied to 'exact' must be variables in 'data'.", call. = FALSE)
-          }
-          exact <- reformulate(exact)
-        }
-        else if (inherits(exact, "formula")) {
-          exact <- update(exact, NULL ~ .)
-        }
-        else {
-          stop("'exact' must be supplied as a character vector of names or a one-sided formula.", call. = FALSE)
-        }
-        exactcovs <- model.frame(exact, data, na.action = "na.pass")
-        if (anyNA(exactcovs)) stop("Missing values are not allowed in the covariates named in 'exact'.", call. = FALSE)
-      }
-
-      if (!is.null(mahvars)) {
-        if (is.character(mahvars)) {
-          if (is.null(data) || !is.data.frame(data)) {
-            stop("If 'mahvars' is specified as strings, a data frame containing the named variables must be supplied to 'data'.", call. = FALSE)
-          }
-          if (!all(mahvars %in% names(data))) {
-            stop("All names supplied to 'mahvars' must be variables in 'data'.", call. = FALSE)
-          }
-          mahvars <- reformulate(mahvars)
-        }
-        else if (inherits(mahvars, "formula")) {
-          mahvars <- update(mahvars, NULL ~ .)
-        }
-        else {
-          stop("'mahvars' must be supplied as a character vector of names or a one-sided formula.", call. = FALSE)
-        }
-        mahcovs <- model.frame(mahvars, data, na.action = "na.pass")
-        if (anyNA(mahcovs)) stop("Missing values are not allowed in the covariates named in 'mahvars'.", call. = FALSE)
-      }
-
-      if (!is.null(antiexact)) {
-        if (is.character(antiexact)) {
-          if (is.null(data) || !is.data.frame(data)) {
-            stop("If 'antiexact' is specified as strings, a data frame containing the named variables must be supplied to 'data'.", call. = FALSE)
-          }
-          if (!all(antiexact %in% names(data))) {
-            stop("All names supplied to 'antiexact' must be variables in 'data'.", call. = FALSE)
-          }
-          antiexact <- reformulate(antiexact)
-        }
-        else if (inherits(antiexact, "formula")) {
-          antiexact <- update(antiexact, NULL ~ .)
-        }
-        else {
-          stop("'antiexact' must be supplied as a character vector of names or a one-sided formula.", call. = FALSE)
-        }
-        antiexactcovs <- model.frame(antiexact, data, na.action = "na.pass")
-        if (anyNA(antiexactcovs)) stop("Missing values are not allowed in the covariates named in 'antiexact'.", call. = FALSE)
-      }
-
-    }
   }
-
   is.full.mahalanobis <- identical(fn1, "distance2mahalanobis")
 
-  #Estimate distance, discard form common support, optionally re-estimate distance
+  #Process exact, mahvars, and antiexact
+  exactcovs <- mahcovs <- antiexactcovs <- NULL
 
-  if (is.null(fn1)) {
-    #Only for cem and exact
+  if (!is.null(exact)) {
+    if (is.character(exact)) {
+      if (is.null(data) || !is.data.frame(data)) {
+        stop("If 'exact' is specified as strings, a data frame containing the named variables must be supplied to 'data'.", call. = FALSE)
+      }
+      if (!all(exact %in% names(data))) {
+        stop("All names supplied to 'exact' must be variables in 'data'.", call. = FALSE)
+      }
+      exact <- reformulate(exact)
+    }
+    else if (inherits(exact, "formula")) {
+      exact <- update(exact, NULL ~ .)
+    }
+    else {
+      stop("'exact' must be supplied as a character vector of names or a one-sided formula.", call. = FALSE)
+    }
+    exactcovs <- model.frame(exact, data, na.action = "na.pass")
+    if (anyNA(exactcovs)) stop("Missing values are not allowed in the covariates named in 'exact'.", call. = FALSE)
+  }
+
+  if (!is.null(mahvars)) {
+    if (is.character(mahvars)) {
+      if (is.null(data) || !is.data.frame(data)) {
+        stop("If 'mahvars' is specified as strings, a data frame containing the named variables must be supplied to 'data'.", call. = FALSE)
+      }
+      if (!all(mahvars %in% names(data))) {
+        stop("All names supplied to 'mahvars' must be variables in 'data'.", call. = FALSE)
+      }
+      mahvars <- reformulate(mahvars)
+    }
+    else if (inherits(mahvars, "formula")) {
+      mahvars <- update(mahvars, NULL ~ .)
+    }
+    else {
+      stop("'mahvars' must be supplied as a character vector of names or a one-sided formula.", call. = FALSE)
+    }
+    mahcovs <- model.frame(mahvars, data, na.action = "na.pass")
+    if (anyNA(mahcovs)) stop("Missing values are not allowed in the covariates named in 'mahvars'.", call. = FALSE)
+  }
+
+  if (!is.null(antiexact)) {
+    if (is.character(antiexact)) {
+      if (is.null(data) || !is.data.frame(data)) {
+        stop("If 'antiexact' is specified as strings, a data frame containing the named variables must be supplied to 'data'.", call. = FALSE)
+      }
+      if (!all(antiexact %in% names(data))) {
+        stop("All names supplied to 'antiexact' must be variables in 'data'.", call. = FALSE)
+      }
+      antiexact <- reformulate(antiexact)
+    }
+    else if (inherits(antiexact, "formula")) {
+      antiexact <- update(antiexact, NULL ~ .)
+    }
+    else {
+      stop("'antiexact' must be supplied as a character vector of names or a one-sided formula.", call. = FALSE)
+    }
+    antiexactcovs <- model.frame(antiexact, data, na.action = "na.pass")
+    if (anyNA(antiexactcovs)) stop("Missing values are not allowed in the covariates named in 'antiexact'.", call. = FALSE)
+  }
+
+  #Estimate distance, discard from common support, optionally re-estimate distance
+  if (is.null(fn1) || fn1 == "distance2mahalanobis") {
+    #No distance measure
     dist.model <- distance <- link <- NULL
-    discarded <- rep(FALSE, length(treat))
   }
   else if (fn1 == "distance2user") {
     dist.model <- link <- NULL
-    discarded <- discard(treat, distance, discard)
-  }
-  else if (fn1 == "distance2mahalanobis") {
-    distance <- link <- dist.model <- NULL
-    discarded <- discard(treat, distance, discard)
   }
   else {
+    if (verbose) {
+      cat("Estimating propensity scores... \n")
+    }
+
     if (!is.null(s.weights)) attr(s.weights, "in_ps") <- !distance %in% c("bart", "randomforest")
 
     #Estimate distance
@@ -188,7 +189,28 @@ matchit <- function(formula, data = NULL, method = "nearest", distance = "glm",
     dist.model <- dist.out$model
     distance <- dist.out$distance
 
-    #Discard
+    #Remove smoothing terms from gam formula
+    if (inherits(dist.model, "gam")) {
+      env <- environment(formula)
+      formula <- mgcv::interpret.gam(formula)$fake.formula
+      environment(formula) <- env
+    }
+  }
+
+  #Process covs
+  covs.formula <- delete.response(terms(formula, data = data))
+  covs <- model.frame(covs.formula, data = data, na.action = "na.pass")
+  for (i in seq_len(ncol(covs))) {
+    if (anyNA(covs[[i]])) stop("Missing values are not allowed in the covariates.", call. = FALSE)
+    if (any(!is.finite(covs[[i]]))) stop("Non-finite values are not allowed in the covariates.", call. = FALSE)
+    if (is.character(covs[[i]])) covs[[i]] <- factor(covs[[i]])
+  }
+
+  #Process discard
+  if (is.null(fn1) || fn1 %in% c("distance2mahalanobis", "distance2user")) {
+    discarded <- discard(treat, distance, discard)
+  }
+  else {
     discarded <- discard(treat, dist.out$distance, discard)
 
     #Optionally reestimate
@@ -205,27 +227,22 @@ matchit <- function(formula, data = NULL, method = "nearest", distance = "glm",
       dist.model <- dist.out$model
       distance[!discarded] <- dist.out$distance
     }
+  }
 
-    #Remove smoothing terms from gam formula
-    if (inherits(dist.model, "gam")) {
-      formula <- mgcv::interpret.gam(formula)$fake.formula
+  #Process caliper
+  calcovs <- NULL
+
+  if (!is.null(caliper)) {
+    caliper <- process.caliper(caliper, method, data, covs, mahcovs, distance, discarded, std.caliper)
+
+    if (!is.null(attr(caliper, "cal.formula"))) {
+      calcovs <- model.frame(attr(caliper, "cal.formula"), data, na.action = "na.pass")
+      if (anyNA(calcovs)) stop("Missing values are not allowed in the covariates named in 'caliper'.", call. = FALSE)
+      attr(caliper, "cal.formula") <- NULL
     }
   }
 
-  covs.formula <- delete.response(terms(formula, data = data))
-  covs <- model.frame(covs.formula, data = data, na.action = "na.pass")
-  if (anyNA(covs)) stop("Missing values are not allowed in the covariates.", call. = FALSE)
-  for (i in which(vapply(covs, is.character, logical(1L)))) covs[[i]] <- factor(covs[[i]])
-
-  caliper <- process.caliper(caliper, method, data, covs, mahcovs, distance, discarded, std.caliper)
-
-  if (!is.null(attr(caliper, "cal.formula"))) {
-    calcovs <- model.frame(attr(caliper, "cal.formula"), data, na.action = "na.pass")
-    if (anyNA(calcovs)) stop("Missing values are not allowed in the covariates named in 'caliper'.", call. = FALSE)
-    attr(caliper, "cal.formula") <- NULL
-  }
-
-  ## matching!
+  #Matching!
   match.out <- do.call(fn2, list(treat = treat, covs = covs, data = data, distance = distance,
                                  discarded = discarded, exact = exact, mahvars = mahvars,
                                  replace = replace, m.order = m.order, caliper = caliper,
@@ -247,20 +264,27 @@ matchit <- function(formula, data = NULL, method = "nearest", distance = "glm",
   X.list[lengths(X.list) == 0] <- NULL
 
   ## putting all the results together
-  match.out$model <- dist.model
-  match.out$X <- do.call("cbind", X.list)
-  match.out$call <- mcall
-  match.out$info <- info
-  match.out$estimand <- estimand
-  match.out$formula <- formula
-  match.out$treat <- treat
-  match.out$distance <- if (!is.null(distance) && !is.matrix(distance)) setNames(distance, names(treat))
-  match.out$discarded <- discarded
-  match.out$s.weights <- s.weights
-  match.out$exact <- exact
-  match.out$mahvars <- mahvars
-  match.out$caliper <- caliper
-  match.out$nn <- nn(treat, match.out$weights, discarded, s.weights)
+  out <- list(
+    match.matrix = match.out[["match.matrix"]],
+    subclass = match.out[["subclass"]],
+    weights = match.out[["weights"]],
+    X = do.call("cbind", X.list),
+    call = mcall,
+    info = info,
+    estimand = estimand,
+    formula = formula,
+    treat = treat,
+    distance = if (!is.null(distance) && !is.matrix(distance)) setNames(distance, names(treat)),
+    discarded = discarded,
+    s.weights = s.weights,
+    exact = exact,
+    mahvars = mahvars,
+    caliper = caliper,
+    nn = nn(treat, match.out$weights, discarded, s.weights),
+    model = dist.model,
+    obj = if (include.obj) match.out[["obj"]]
+  )
 
-  return(match.out)
+  class(out) <- class(match.out)
+  return(out)
 }
